#ifndef CPARCELLEPERSISTANTE_H
#define CPARCELLEPERSISTANTE_H

#include "CPersistant.h"

namespace persistance {

// ________________________________________________________________________________________
/** \class CParcellePersistante
 * Classe CParcellePersistante basée sur une stratégie de stockage <TStrategie> pour sauvegarder des parcelles : \n
 * CParcellePersistante<TStrategie> est une classe générique concrète, basée sur la classe générique de base CPersistant<TStrategie> \n
 * - La classe redéfinie les méthodes CParcellePersistante::ecrire et CParcellePersistante::nomclasse \n
 * - Elle définie un constructeur flux pour la lecture \n
 * - Elle définie une méthode statique CParcellePersistante::constructeur pour référencer la classe \n
 * - Elle prépare une méthode CParcellePersistante::referencer à appeler pour son référencement \n
 * Elle est basée sur une stratégie de stockage (exemple CBinaire).
 */template <class TStrategie>
class CParcellePersistante : public CPersistant<TStrategie> {

	public:
		/** Constructeur CParcellePersistante::CParcellePersistante \n
		 * Initialise la classe de base CPersistant et la parcelle. \n
		 * \param kprIDU Identifiant de la parcelle.
		 * \param kpvSurface Surface de la parcelle.
		 * \param pvPersistant précise si la classe doit être ou non persistante.
		 * \param pvAlloue précise si la classe est allouée. Permettra au destructeur de la classe de base de libérer la mémoire.
		 */
		CParcellePersistante (const std::string& kprIDU, const double kpvSurface, bool pvPersistant, bool pvAlloue);

		/** Destructeur CParcellePersistante::~CParcellePersistante \n
		 * Détruit une parcelle persistante \n
		 */
		virtual ~CParcellePersistante () {};

		/** Méthode CParcellePersistante::ecrire : enregistre l'objet dans un flux. \n
		 * Commence par appeler l'écriture de la classe de base (obligatoire). \n
		 * Puis écrit métadonnées et attributs de la parcelle selon la stratégie de stockage.
		 * \param prFlux : Flux dans lequel écrire la parcelle.
		 */
		virtual void ecrire (std::ostream& prFlux) const;

		/** Méthode virtuelle CParcellePersistante::nomClasse : Cette méthode renvoie le nom de la classe. \n
		 * Le nom de classe renvoyé intègre le schéma et la stratégie (exemple : "persistance::CParcellePersistante<CBinaire>"). \n
		 * La stratégie doit donc pouvoir délivrer son propre nom.
		 * @return renvoie le nom de la classe.
		 */
		virtual std::string nomClasse () const { return smvNomClasse; };

		/** Méthode statique CParcellePersistante::constructeur : Cette méthode construit par allocation un objet CParcellePersistante. \n
		 * L'objet est créé et initialisé par appel au constructeur flux. \n
		 * @return renvoie un pointeur sur l'objet persistant alloué.
		 */
		static CBasePersistant* constructeur (std::istream& prFlux) { return new CParcellePersistante<TStrategie> (prFlux); };

		/** Méthode statique CParcellePersistante::referencer : Cette méthode référence la classe en associant le nom de classe avec sa méthode CParcellePersistante::constructeur. \n
		 * Cette méthode doit être obligatoirement appelée par tout programme intéressé pour invoquer correctement la lecture. \n
		 */
		static void referencer ();

	protected:

		/** Constructeur flux CParcellePersistante::CParcellePersistante \n
		 * Initialise l'objet à partir des données du flux. \n
		 * Il faut commencer par lire les données de la classe de base avec le constructeur flux hérité \n
		 * L'objet étant alloué et persistant, les attributs correspondants sont activés. \n
		 * \param prFlux Flux dans lequel lire la parcelle.
		 */
		CParcellePersistante (std::istream& prFlux);

		CParcellePersistante (const CParcellePersistante& kprModele) = delete;
		CParcellePersistante& operator= (const CParcellePersistante& kprModele) = delete;

	private:
		std::string mvIDU; /**< Identifiant de parcelle */
		double mvSurface; /**< Surface de la parcelle en m² */

		static std::string smvNomClasse;
	};
};

#endif // CPARCELLEPERSISTANTE_H
